@@ -3,14 +3,14 @@ EvenAge - A small-surface, high-abstraction distributed agent framework.
 
 EvenAge provides:
 - Minimal API with @actor and @tool decorators
-- Pluggable backends (queue, database, LLM, cache, vector)
+- Pluggable backends (queue, database, LLM, cache)
 - Internal observability (no Jaeger/Prometheus needed)
 - Async-first architecture
 - Docker-native execution
 - Real distributed runtime with message bus
 """
 
-__version__ = "0.7.8"
+__version__ = "0.8.4"
 
 # Core exports
 from evenage.core import (
@@ -24,11 +24,9 @@ from evenage.core import (
     MemoryBus,
     RedisBus,
     ToolRegistry,
-    actor,
     discover_tools,
     run_agent,
     tool,
-    Workflow,
 )
 
 
@@ -40,7 +38,6 @@ def Agent(
     backstory: str | None = None,
     tools: "ToolRegistry | list[object] | None" = None,
     llm: object | None = None,
-    llm_fallbacks: list[object] | None = None,
     queue: object | None = None,
     instructions: str | None = None,
     is_coordinator: bool | None = None,
@@ -70,7 +67,6 @@ def Agent(
         cfg,
         tools=tools,  # CoreAgent accepts ToolRegistry or list of callables
         llm=llm,
-        llm_fallbacks=llm_fallbacks,
         queue=queue,
         instructions=instructions,
         is_coordinator=is_coordinator,
@@ -87,9 +83,7 @@ __all__ = [
     "Agent",
     "AgentRunner",
     "WorkerRunner",
-    "actor",
     "tool",
-    "Workflow",
     "run_agent",
     # Configuration
     "EvenAgeConfig",
