@@ -45,6 +45,8 @@ class SubmitPromptRequest(BaseModel):
     """Request to submit a new prompt execution."""
     prompt_name: str
     inputs: dict[str, Any]
+    mode: str = "single"  # "single" or "sequential"
+    agents: list[str] | None = None  # For sequential mode: ordered list of agents
 
 
 class CancelPromptResponse(BaseModel):
@@ -107,6 +109,65 @@ async def lifespan(app: FastAPI):
     logger.info("EvenAge API server shutting down")
 
 
+async def _run_sequential_chain(runner: AgentRunner, agent_names: list[str], inputs: dict, job_id: str) -> dict:
+    """
+    Run agents sequentially, piping output from one to the next.
+    
+    Args:
+        runner: AgentRunner instance
+        agent_names: List of agent names in execution order
+        inputs: Initial inputs
+        job_id: Job ID to use for all agents in the chain
+    
+    Returns:
+        Final result dict with sequential execution metadata
+    """
+    current_input = inputs
+    last_result = None
+    results = []
+    
+    for agent_name in agent_names:
+        try:
+            result = await runner.run(current_input, agent_name=agent_name, job_id=job_id)
+            
+            if result.get("status") == "error":
+                return {
+                    "status": "error",
+                    "error": f"Sequential chain failed at {agent_name}: {result.get('error')}",
+                    "agent": agent_name,
+                    "results": results
+                }
+            
+            # Extract result for next agent
+            last_result = result.get("result", result)
+            results.append({
+                "agent": agent_name,
+                "result": last_result
+            })
+            
+            # Next agent receives previous output
+            if isinstance(last_result, dict):
+                current_input = last_result
+            else:
+                current_input = {"input": last_result, "text": str(last_result)}
+        
+        except Exception as e:
+            return {
+                "status": "error",
+                "error": f"Sequential chain failed at {agent_name}: {e}",
+                "agent": agent_name,
+                "results": results
+            }
+    
+    return {
+        "status": "success",
+        "result": last_result,
+        "mode": "sequential",
+        "agents": agent_names,
+        "results": results
+    }
+
+
 # Create FastAPI app
 app = FastAPI(
     title="EvenAge API",
@@ -224,6 +285,13 @@ async def get_job(job_id: str):
         if not result:
             raise HTTPException(status_code=404, detail="Job not found")
 
+        # Aggregate token usage
+        usage = {}
+        try:
+            usage = database.get_token_usage(job_id)
+        except Exception:
+            usage = {}
+
         return {
             "job_id": job_id,
             "status": result.get("status"),
@@ -232,7 +300,8 @@ async def get_job(job_id: str):
             "outputs": result.get("outputs"),
             "error": result.get("error"),
             "created_at": result.get("created_at"),
-            "completed_at": result.get("completed_at")
+            "completed_at": result.get("completed_at"),
+            "token_usage": usage,
         }
 
     except HTTPException:
@@ -311,6 +380,7 @@ async def submit_prompt(request: SubmitPromptRequest):
     - Creates a prompt record immediately
     - Kicks off background agent execution using the same prompt_id as job_id
     - Returns promptly with status 'running' and the prompt_id
+    - Supports sequential mode: runs multiple agents in order, piping output
     """
     try:
         # Create prompt record first and mark as running
@@ -320,11 +390,17 @@ async def submit_prompt(request: SubmitPromptRequest):
         async def _run_in_background(pid: str):
             try:
                 runner = AgentRunner(env_config=config)
-                result = await runner.run(
-                    inputs=request.inputs,
-                    agent_name=request.prompt_name,
-                    job_id=pid,
-                )
+                
+                # Sequential mode: run multiple agents in order
+                if request.mode == "sequential" and request.agents and len(request.agents) > 1:
+                    result = await _run_sequential_chain(runner, request.agents, request.inputs, pid)
+                else:
+                    # Single agent mode (default)
+                    result = await runner.run(
+                        inputs=request.inputs,
+                        agent_name=request.prompt_name,
+                        job_id=pid,
+                    )
 
                 # Map status to prompt status
                 status = result.get("status", "failed")
@@ -348,7 +424,9 @@ async def submit_prompt(request: SubmitPromptRequest):
         return {
             "prompt_id": prompt_id,
             "status": "running",
-            "agent": request.prompt_name
+            "agent": request.prompt_name,
+            "mode": request.mode,
+            "agents": request.agents if request.mode == "sequential" else [request.prompt_name]
         }
 
     except Exception as e:
@@ -373,6 +451,13 @@ async def get_prompt(prompt_id: str):
         if not result:
             raise HTTPException(status_code=404, detail="Prompt execution not found")
 
+        # Aggregate token usage for this prompt/job
+        usage = {}
+        try:
+            usage = database.get_token_usage(prompt_id)
+        except Exception:
+            usage = {}
+
         return {
             "prompt_id": prompt_id,
             "status": result.get("status"),
@@ -381,7 +466,8 @@ async def get_prompt(prompt_id: str):
             "outputs": result.get("outputs"),
             "error": result.get("error"),
             "created_at": result.get("created_at"),
-            "completed_at": result.get("completed_at")
+            "completed_at": result.get("completed_at"),
+            "token_usage": usage,
         }
 
     except HTTPException:
@@ -435,19 +521,20 @@ async def list_agents():
         List of agents with metadata
     """
     try:
-        agents = await message_bus.get_registered_agents()
+        # Query database for agent metadata (registered by workers)
+        agents_list = database.list_agents()
 
         return {
             "agents": [
                 AgentInfo(
-                    name=name,
-                    role=meta.get("role", "unknown"),
-                    goal=meta.get("goal", ""),
-                    status=meta.get("status", "unknown"),
-                    tools=meta.get("tools", []),
-                    last_seen=meta.get("last_seen")
+                    name=agent["agent_name"],
+                    role=agent["metadata"].get("role", "unknown"),
+                    goal=agent["metadata"].get("goal", ""),
+                    status=agent["metadata"].get("status", "unknown"),
+                    tools=agent["metadata"].get("tools", []),
+                    last_seen=agent.get("last_seen")
                 ).dict()
-                for name, meta in agents.items()
+                for agent in agents_list
             ]
         }
 
@@ -698,7 +785,7 @@ async def get_metrics():
             "active_agents_count": metrics.get("active_agents_count", 0),
             "cache_hits": metrics.get("cache_hits", 0),
             "tool_calls": metrics.get("tool_calls", 0),
-            "timestamp": datetime.utcnow().isoformat()
+            "timestamp": datetime.utcnow().isoformat() + "Z"
         }
 
     except Exception as e:
@@ -799,6 +886,18 @@ async def websocket_prompt_traces(websocket: WebSocket, prompt_id: str):
     
     # Track last seen trace ID to avoid duplicates
     last_trace_id = 0
+    # Track open delegation windows to annotate child traces with parent_trace_id
+    # Shape: { parent_trace_id: { 'agent': str|None, 'start': datetime, 'end': datetime|None } }
+    open_delegations: dict[int, dict[str, Any]] = {}
+
+    def _parse_ts(ts: str | None) -> datetime | None:
+        if not ts:
+            return None
+        try:
+            # Support "Z" timezone by replacing with +00:00 for fromisoformat
+            return datetime.fromisoformat(ts.replace("Z", "+00:00")).replace(tzinfo=None)
+        except Exception:
+            return None
     
     try:
         while True:
@@ -814,6 +913,57 @@ async def websocket_prompt_traces(websocket: WebSocket, prompt_id: str):
                 
                 # Send new traces
                 for trace in new_traces:
+                    # Heuristically detect delegation start/end to annotate children
+                    try:
+                        event_type = trace.get('event_type')
+                        payload = trace.get('payload') or {}
+                        tool_name = (payload.get('tool') or '').lower()
+
+                        # Delegation start (both single and parallel)
+                        if event_type == 'tool_call' and tool_name in {
+                            'delegate_parallel', 'system_delegate_parallel',
+                            'delegate_task', 'system_delegate_task'
+                        }:
+                            parent_id = trace.get('id')
+                            params = payload.get('params') or {}
+                            delegated_agent = params.get('agent') or params.get('agent_name')
+                            open_delegations[int(parent_id)] = {
+                                'agent': delegated_agent,
+                                'start': _parse_ts(trace.get('timestamp')),
+                                'end': None,
+                            }
+
+                        # Delegation end (close the most recent open window if any)
+                        if event_type == 'tool_result' and tool_name in {
+                            'delegate_parallel', 'system_delegate_parallel',
+                            'delegate_task', 'system_delegate_task'
+                        }:
+                            end_ts = _parse_ts(trace.get('timestamp'))
+                            # Close all open windows that have no end yet (handles overlapping conservatively)
+                            for k, v in list(open_delegations.items()):
+                                if v.get('end') is None:
+                                    v['end'] = end_ts
+                                    open_delegations[k] = v
+
+                        # Annotate child traces that belong to an open delegation window
+                        if event_type not in {'tool_call', 'tool_result'}:
+                            t_ts = _parse_ts(trace.get('timestamp'))
+                            agent_name = trace.get('agent_name')
+                            parent_to_use: int | None = None
+                            for pid, window in open_delegations.items():
+                                if window.get('agent') and agent_name == window['agent']:
+                                    start_ok = (window.get('start') is None) or (t_ts and window['start'] and t_ts >= window['start'])
+                                    end_ok = (window.get('end') is None) or (t_ts and window['end'] and t_ts <= window['end'])
+                                    if start_ok and end_ok:
+                                        parent_to_use = pid
+                                        break
+                            if parent_to_use is not None:
+                                # Attach parent_trace_id for the UI to nest this event
+                                trace = {**trace, 'parent_trace_id': parent_to_use}
+                    except Exception:
+                        # Best-effort enrichment; never break the stream
+                        pass
+
                     await websocket.send_json({
                         "type": "trace",
                         "data": trace
