@@ -187,17 +187,17 @@ def _generate_default_agent(project_path: Path) -> None:
     default_agents = [
         {
             "name": "coordinator",
-            "role": "Orchestrator & Task Planner",
-            "goal": "Break down complex tasks into subtasks and coordinate their execution across specialized agents",
-            "backstory": "Experienced project manager with a talent for decomposition and delegation.",
+            "role": "Orchestration Coordinator & Task Manager",
+            "goal": "Analyze complex user requests, break them into specific subtasks, delegate each subtask to the appropriate specialist agent, and synthesize their responses into a comprehensive final answer",
+            "backstory": "An experienced orchestrator who excels at task decomposition and delegation. You never perform research yourself - instead, you coordinate a team of specialists. You understand that effective delegation requires clear, specific instructions and that your role is to combine specialist outputs into coherent, complete responses for users.",
             "is_coordinator": True,
-            "available_agents": ["researcher", "analyst", "writer"],
+            "available_agents": ["researcher"],
         },
         {
             "name": "researcher",
-            "role": "Research Agent",
-            "goal": "Research topics on the web and summarize findings concisely.",
-            "backstory": "A helpful researcher that gathers and condenses information from reliable web sources.",
+            "role": "Web Research Specialist & Information Analyst",
+            "goal": "Conduct thorough web research using available tools (web_search, scrape_url, extract_links) to find accurate, relevant information and deliver comprehensive, well-sourced answers",
+            "backstory": "A meticulous research specialist with expertise in web searching, content extraction, and information synthesis. You have direct access to powerful web tools and know how to use them effectively. Your strength is finding high-quality sources, extracting key information, and presenting findings clearly with proper citations. You prioritize accuracy and always verify information across multiple sources when possible.",
             "is_coordinator": False,
             "available_agents": [],
         }
@@ -226,6 +226,7 @@ def _generate_default_agent(project_path: Path) -> None:
             "fallback_model": "gpt-3.5-turbo",
             "temperature": 0.3 if agent_config.get("is_coordinator") else 0.2,
             "max_retries": 3,
+            "max_iterations": 5 if agent_config.get("is_coordinator") else 15,
             "timeout": 600,
             "instructions": (
                 "When you receive a task: 1) Analyze 2) Plan 3) Execute using tools 4) Synthesize results."
