@@ -45,6 +45,77 @@ class AgentMemory:
         return self.database.memory_list(self.agent_name, self.job_id)
 
 
+@dataclass
+class PausedExecutionContext:
+    """
+    Saved execution context for paused agents waiting on delegation.
+    
+    This enables non-blocking delegation: when an agent delegates a task,
+    it saves its state and returns control to the orchestrator. When the
+    delegated task completes, the orchestrator resumes the parent agent
+    from this saved context.
+    """
+    parent_task_id: str  # The task_id of the parent agent's execution
+    parent_agent_name: str  # Name of the agent that delegated
+    delegation_task_id: str  # Task_id of the delegated subtask
+    delegated_agent_name: str  # Target agent for delegation
+    delegated_task: dict  # The task payload sent to delegated agent
+    
+    # Current execution state
+    task: dict  # Original task inputs
+    history: list[dict]  # Conversation history up to delegation point
+    iteration: int  # Which iteration was paused
+    max_iterations: int  # Max iterations allowed
+    max_tool_calls: int | None  # Max tool calls allowed (None = unlimited)
+    
+    # Deduplication tracking
+    seen_queries: set[str]  # Query deduplication cache
+    seen_delegations: dict[str, set[str]]  # Delegation deduplication cache
+    
+    # Timestamps
+    paused_at: str  # ISO timestamp when paused
+    timeout_seconds: int  # How long to wait for delegation response
+    
+    def to_dict(self) -> dict:
+        """Serialize to dict for storage."""
+        return {
+            "parent_task_id": self.parent_task_id,
+            "parent_agent_name": self.parent_agent_name,
+            "delegation_task_id": self.delegation_task_id,
+            "delegated_agent_name": self.delegated_agent_name,
+            "delegated_task": self.delegated_task,
+            "task": self.task,
+            "history": self.history,
+            "iteration": self.iteration,
+            "max_iterations": self.max_iterations,
+            "max_tool_calls": self.max_tool_calls,
+            "seen_queries": list(self.seen_queries),
+            "seen_delegations": {k: list(v) for k, v in self.seen_delegations.items()},
+            "paused_at": self.paused_at,
+            "timeout_seconds": self.timeout_seconds,
+        }
+    
+    @classmethod
+    def from_dict(cls, data: dict) -> "PausedExecutionContext":
+        """Deserialize from dict."""
+        return cls(
+            parent_task_id=data["parent_task_id"],
+            parent_agent_name=data["parent_agent_name"],
+            delegation_task_id=data["delegation_task_id"],
+            delegated_agent_name=data["delegated_agent_name"],
+            delegated_task=data["delegated_task"],
+            task=data["task"],
+            history=data["history"],
+            iteration=data["iteration"],
+            max_iterations=data["max_iterations"],
+            max_tool_calls=data.get("max_tool_calls"),
+            seen_queries=set(data.get("seen_queries", [])),
+            seen_delegations={k: set(v) for k, v in data.get("seen_delegations", {}).items()},
+            paused_at=data["paused_at"],
+            timeout_seconds=data.get("timeout_seconds", 60),
+        )
+
+
 class Agent:
     """
     Base Agent class with full runtime capabilities.
@@ -52,12 +123,24 @@ class Agent:
     Provides:
     - Planning and execution
     - Tool calling with validation and caching
-    - Delegation to other agents via message bus
+    - Non-blocking delegation with pause/resume orchestration
     - Memory storage
     - Auto-registration on message bus
     - Trace logging to database
     
-    Subclasses should override plan() method for custom planning logic.
+    Delegation Model:
+    - Modern autonomous agents use NON-BLOCKING delegation
+    - When delegating, the agent saves its state and returns status="paused"
+    - The orchestrator calls resume_paused_execution() when delegation completes
+    - This enables event-driven, scalable multi-agent workflows
+    
+    Backward Compatibility:
+    - Custom plan() methods can still use blocking delegation via wait_for_response
+    - System tools like delegate_task support wait_for_response parameter
+    - All existing agent code continues to work unchanged
+    
+    Subclasses should override plan() method for custom planning logic,
+    or use the default autonomous_run() for intelligent LLM-driven execution.
     """
 
     # Class-level metadata (set by @actor decorator or subclass)
@@ -71,7 +154,6 @@ class Agent:
         tools: ToolRegistry | list[Any] | None = None,
         *,
         llm: Any | None = None,
-        llm_fallbacks: list[Any] | None = None,
         queue: Any | None = None,
         instructions: str | None = None,
         is_coordinator: bool | None = None,
@@ -113,12 +195,11 @@ class Agent:
 
         self.database = self.factory.create_database_backend()
 
-        # LLM override with fallback support
+        # LLM override
         if llm is not None:
             self.llm = llm
         else:
             self.llm = self.factory.create_llm_backend()
-        self._llm_fallbacks: list[Any] = list(llm_fallbacks or [])
         # Default LLM params (temperature, max_tokens, etc.) optionally provided by llm wrapper
         self._llm_params: dict = {}
         try:
@@ -166,6 +247,7 @@ class Agent:
         self._heartbeat_task: asyncio.Task | None = None
 
         # Coordinator support (allow explicit override)
+        # hierarchal delegation setup
         self.is_coordinator = (
             bool(is_coordinator)
             if is_coordinator is not None
@@ -184,40 +266,36 @@ class Agent:
         self._extra_instructions: str | None = (instructions or None)
 
     async def _llm_generate(self, prompt: str, system: str | None = None, **kwargs) -> str:
-        """Generate with primary LLM and fallbacks with default params merged.
+        """Generate with primary LLM.
 
         If the backend supports generate_with_usage, capture token usage and emit a trace
         event 'llm_usage' with provider/model and token counts for per-job accounting.
         """
         params = {**self._llm_params, **(kwargs or {})}
-        errors: list[str] = []
-        backends = [self.llm] + [b for b in self._llm_fallbacks if b is not None]
-        for backend in backends:
-            try:
-                # Prefer usage-aware API if available
-                if hasattr(backend, "generate_with_usage"):
-                    text, usage = await backend.generate_with_usage(prompt, system=system, **params)  # type: ignore[attr-defined]
-                    # Trace usage if available
-                    try:
-                        if isinstance(usage, dict):
-                            payload = {
-                                "provider": usage.get("provider"),
-                                "model": usage.get("model"),
-                                "prompt_tokens": usage.get("prompt_tokens"),
-                                "completion_tokens": usage.get("completion_tokens"),
-                                "total_tokens": usage.get("total_tokens"),
-                                "worker": self.config.name,
-                            }
-                            self._trace(self.current_job_id or "unknown", self.config.name, "llm_usage", payload)
-                    except Exception:
-                        pass
-                    return text
-                # Fallback to text-only generation
-                return await backend.generate(prompt, system=system, **params)
-            except Exception as e:
-                errors.append(str(e))
-                continue
-        raise RuntimeError(f"All LLM backends failed: {' | '.join(errors[:3])}")
+        
+        try:
+            # Prefer usage-aware API if available
+            if hasattr(self.llm, "generate_with_usage"):
+                text, usage = await self.llm.generate_with_usage(prompt, system=system, **params)  # type: ignore[attr-defined]
+                # Trace usage if available
+                try:
+                    if isinstance(usage, dict):
+                        payload = {
+                            "provider": usage.get("provider"),
+                            "model": usage.get("model"),
+                            "prompt_tokens": usage.get("prompt_tokens"),
+                            "completion_tokens": usage.get("completion_tokens"),
+                            "total_tokens": usage.get("total_tokens"),
+                            "worker": self.config.name,
+                        }
+                        self._trace(self.current_job_id or "unknown", self.config.name, "llm_usage", payload)
+                except Exception:
+                    pass
+                return text
+            # Fallback to text-only generation
+            return await self.llm.generate(prompt, system=system, **params)
+        except Exception as e:
+            raise RuntimeError(f"LLM generation failed: {str(e)}")
 
     def _register_system_tools(self):
         """Register built-in system tools for delegation and artifacts."""
@@ -230,19 +308,28 @@ class Agent:
         # Disallow any system tool that could bypass delegation safeguards
         reserved = {"delegate", "publish", "publish_task"}
 
-        for name, func in system_tools.items():
+        for name, tool_data in system_tools.items():
             # Skip reserved/unsafe tool names
             if name in reserved:
                 continue
+            
+            # Unpack tool data - can be either (func, aliases) or just func
+            if isinstance(tool_data, tuple):
+                func, aliases = tool_data
+            else:
+                func = tool_data
+                aliases = []
+            
             # Register each system tool with the explicit provided name,
             # not the callable's __name__, so both prefixed and alias names exist.
             if not self.tools.has(name):
                 try:
                     desc = (func.__doc__ or "").strip().split("\n")[0]
-                    self.tools.register(_Tool(name=name, func=func, description=desc))
+                    tool = _Tool(name=name, func=func, description=desc)
+                    self.tools.register(tool, name=name, aliases=aliases)
                 except Exception:
                     # Fallback to direct registration
-                    self.tools.register(func)
+                    self.tools.register(func, name=name, aliases=aliases)
 
     async def connect_bus(self) -> None:
         """
@@ -373,7 +460,14 @@ class Agent:
             
             if plan_method is base_plan_method:
                 # Use autonomous agent loop (intelligent, LLM-driven)
-                result = await self.autonomous_run(task)
+                # Extract execution limits from task (if provided by user)
+                max_iterations = task.get("max_iterations")
+                max_tool_calls = task.get("max_tool_calls")
+                result = await self.autonomous_run(
+                    task, 
+                    max_iterations=max_iterations,
+                    max_tool_calls=max_tool_calls
+                )
             else:
                 # Use custom plan (backward compatibility)
                 plan = self.plan(task)
@@ -420,7 +514,7 @@ class Agent:
             return {
                 "status": "success",
                 "result": result.get("result") if isinstance(result, dict) else result,
-                "job_id": execution_id,  # Keep for backward compatibility
+                "job_id": execution_id,  # Keep for backward compatibility (no clue why this is here maybe we'll find out one day)
                 "prompt_id": execution_id  # Add new terminology
             }
 
@@ -470,8 +564,8 @@ class Agent:
                 result = await self._execute_delegate_step(step, context)
             elif action == "llm":
                 result = await self._execute_llm_step(step, context)
-            elif action == "custom":
-                result = await self._execute_custom_step(step, context)
+            # elif action == "custom":
+            #     result = await self._execute_custom_step(step, context)
             else:
                 result = {"error": f"Unknown action: {action}"}
 
@@ -489,13 +583,23 @@ class Agent:
         return await self.call_tool(tool_name, params, use_cache=use_cache)
 
     async def _execute_delegate_step(self, step: dict, context: dict) -> Any:
-        """Execute a delegation step."""
+        """
+        Execute a delegation step.
+        
+        This method is for backward compatibility with custom plan() implementations.
+        
+        For modern autonomous_run() agents, delegation is non-blocking and uses
+        pause/resume orchestration automatically.
+        
+        For legacy custom plans, set wait=True to block until response (old behavior).
+        Set wait=False to delegate without waiting (fire-and-forget).
+        """
         agent_name = step.get("agent")
         payload = step.get("payload", {})
 
         task_id = await self.delegate(agent_name, payload)
 
-        # Optionally wait for response (blocking)
+        # Optionally wait for response (BLOCKING - backward compatibility mode)
         wait = step.get("wait", False)
         if wait:
             timeout = step.get("timeout", 30)
@@ -520,9 +624,9 @@ class Agent:
         response = await self._llm_generate(prompt, system=system)
         return {"llm_response": response}
 
-    async def _execute_custom_step(self, step: dict, context: dict) -> Any:
-        """Execute a custom step (override in subclass)."""
-        return {"error": "Custom step not implemented"}
+    # async def _execute_custom_step(self, step: dict, context: dict) -> Any:
+    #     """Execute a custom step (override in subclass)."""
+    #     return {"error": "Custom step not implemented"}
 
     async def call_tool(
         self,
@@ -720,21 +824,64 @@ class Agent:
         except Exception:
             pass
 
-    async def autonomous_run(self, task: dict, max_iterations: int | None = None) -> dict:
+    async def autonomous_run(
+        self, 
+        task: dict, 
+        max_iterations: int | None = None,
+        max_tool_calls: int | None = None
+    ) -> dict:
         """
         Run agent autonomously using ReAct-style loop.
         
         The LLM decides what actions to take (use tools, delegate, or finish).
         This is the simple, intuitive way to create agents - no manual planning needed.
         
+        Supports non-blocking delegation: when delegating to another agent, this method
+        will save execution state and return with status="paused". The orchestrator
+        should call resume_paused_execution() when the delegation completes.
+        
         Args:
             task: Task inputs
-            max_iterations: Override max iterations from config
+            max_iterations: Maximum number of think-act cycles (each iteration can be tool/delegate/think)
+            max_tool_calls: Maximum number of successful tool calls before stopping (None = unlimited)
         
         Returns:
-            Final result dict
+            Final result dict (may have status="paused" if waiting on delegation)
         """
-        max_iter = max_iterations or self.config.max_iterations or 5  # Reduced to 5 with forced finish at 4 tools
+        max_iter = max_iterations or self.config.max_iterations or 10
+        max_tools = max_tool_calls  # None means unlimited
+        
+        # Start fresh execution
+        return await self._autonomous_run_loop(
+            task=task,
+            history=[],
+            start_iteration=0,
+            max_iterations=max_iter,
+            max_tool_calls=max_tools,
+            seen_queries=set(),
+            seen_delegations={}
+        )
+    
+    async def _autonomous_run_loop(
+        self,
+        task: dict,
+        history: list[dict],
+        start_iteration: int,
+        max_iterations: int,
+        max_tool_calls: int | None,
+        seen_queries: set[str],
+        seen_delegations: dict[str, set[str]]
+    ) -> dict:
+        """
+        Internal autonomous execution loop.
+        
+        Extracted from autonomous_run to support resumption from saved state.
+        This method contains the actual ReAct loop logic.
+        
+        Returns status="paused" if delegation occurs (non-blocking model).
+        """
+        max_iter = max_iterations
+        max_tools = max_tool_calls
         
         # Build context
         task_description = (
@@ -760,17 +907,8 @@ class Agent:
                 ]
             except Exception:
                 pass
-        
-        # Track conversation history
-        history = []
-        
-        # Query deduplication: track normalized tool calls to prevent redundant searches
-        seen_queries = set()
-        
-        # Delegation deduplication: track tasks delegated to each agent to prevent redundant delegations
-        seen_delegations = {}  # Map: agent_name -> set of normalized tasks
 
-        for iteration in range(max_iter):
+        for iteration in range(start_iteration, max_iter):
             # Cooperative cancellation: check before each iteration
             try:
                 if self.current_job_id and hasattr(self.bus, "is_canceled"):
@@ -792,6 +930,7 @@ class Agent:
                 # If cancel check fails, continue execution
                 pass
             # Build system prompt
+            # user doesnt need to specify available tools/agents - we provide them
             system_prompt = self._build_autonomous_system_prompt(
                 available_tools, 
                 available_agents,
@@ -1011,47 +1150,59 @@ class Agent:
                 # Record this delegation
                 seen_delegations[agent_name].add(normalized_task)
 
+                # NON-BLOCKING DELEGATION: Save context and pause execution
                 try:
-                    task_id = await self.delegate(agent_name, {"query": delegate_task})
-                    logger.info("[%s] delegation task_id=%s; waiting for response...", self.config.name, task_id)
+                    # Publish delegation task
+                    delegation_task_payload = {"query": delegate_task}
+                    delegation_task_id = await self.delegate(agent_name, delegation_task_payload)
+                    logger.info(
+                        "[%s] delegation task_id=%s to %s (NON-BLOCKING); pausing execution...",
+                        self.config.name,
+                        delegation_task_id,
+                        agent_name
+                    )
                     
-                    # Wait for result
+                    # Create paused context
+                    from datetime import datetime as _dt
+                    paused_at = _dt.utcnow().isoformat() + "Z"
                     timeout = action.get("timeout", 60)
-                    result = await self.bus.wait_for_response(task_id, timeout)
-                    logger.info("[%s] delegation result received: %s", self.config.name, str(result)[:200])
                     
-                    # CRITICAL: Check if delegated task actually succeeded
-                    # Don't blindly retry if the delegated agent couldn't complete it
-                    result_status = None
-                    result_data = None
+                    paused_context = PausedExecutionContext(
+                        parent_task_id=self.current_job_id,
+                        parent_agent_name=self.config.name,
+                        delegation_task_id=delegation_task_id,
+                        delegated_agent_name=agent_name,
+                        delegated_task=delegation_task_payload,
+                        task=task,
+                        history=history,
+                        iteration=iteration,
+                        max_iterations=max_iter,
+                        max_tool_calls=max_tools,
+                        seen_queries=seen_queries,
+                        seen_delegations=seen_delegations,
+                        paused_at=paused_at,
+                        timeout_seconds=timeout
+                    )
                     
-                    if isinstance(result, dict):
-                        result_status = result.get("status")
-                        result_data = result.get("result") or result.get("error")
+                    # Save paused context
+                    await self._save_paused_context(paused_context)
                     
-                    history.append({
-                        "action": "delegate",
-                        "agent": agent_name,
-                        "task": delegate_task,
-                        "result": result,
-                        "status": result_status
-                    })
-                    
-                    # If delegated task failed or was incomplete, finish with that information
-                    # Don't keep retrying - it will just fail again
-                    if result_status in ["error", "incomplete"]:
-                        logger.warning(
-                            "[%s] Delegated task to %s returned status=%s, stopping to avoid retry loop",
-                            self.config.name,
-                            agent_name,
-                            result_status
-                        )
-                        return {
-                            "status": "error",
-                            "error": f"Delegated task to {agent_name} failed: {result_data}",
-                            "iterations": iteration + 1,
-                            "history": history
-                        }
+                    # Return PAUSED status to orchestrator
+                    logger.info(
+                        "[%s] Execution PAUSED at iteration %d, waiting for delegation_task_id=%s",
+                        self.config.name,
+                        iteration,
+                        delegation_task_id
+                    )
+                    return {
+                        "status": "paused",
+                        "message": f"Execution paused, waiting for delegation to {agent_name}",
+                        "delegation_task_id": delegation_task_id,
+                        "delegated_agent": agent_name,
+                        "parent_task_id": self.current_job_id,
+                        "iteration": iteration,
+                        "prompt_id": self.current_job_id
+                    }
                     
                 except Exception as e:
                     logger.exception("[%s] delegation error: %s", self.config.name, e)
@@ -1076,50 +1227,31 @@ class Agent:
                     "thought": response
                 })
             
-            # Sufficiency heuristic: if we have enough tool results, suggest finishing
-            # Count successful tool calls with actual results (excluding duplicates)
-            successful_tool_calls = [
-                h for h in history 
-                if h.get("action") == "tool" 
-                and "result" in h 
-                and h["result"] not in [None, ""]
-                and not str(h["result"]).startswith("Skipped:")
-                and "error" not in h
-            ]
-            
-            # CRITICAL: Force finish after 4 successful tool calls to prevent infinite loops
-            if len(successful_tool_calls) >= 4:
-                logger.warning(
-                    "[%s] FORCING FINISH: %d successful tool calls completed. "
-                    "Preventing excessive iterations.",
-                    self.config.name,
-                    len(successful_tool_calls)
-                )
-                return {
-                    "status": "success",
-                    "result": "Task completed with sufficient information gathered. Agent was stopped to prevent excessive iterations.",
-                    "iterations": iteration + 1,
-                    "history": history,
-                    "forced_completion": True
-                }
-            
-            # Soft hint after 2 successful calls
-            if len(successful_tool_calls) >= 2:
-                logger.info(
-                    "[%s] Sufficiency check: %d successful tool calls completed. "
-                    "Strongly suggesting to finish the task in next iteration.",
-                    self.config.name,
-                    len(successful_tool_calls)
-                )
-                # Add a STRONG hint in history for next LLM call
-                history.append({
-                    "action": "system_hint",
-                    "message": (
-                        f"IMPORTANT: You have gathered {len(successful_tool_calls)} pieces of information. "
-                        "This is sufficient for most tasks. You MUST finish with a comprehensive answer in the next response. "
-                        "Do NOT make additional tool calls unless absolutely critical information is missing."
+            # Check if we've hit max_tool_calls limit
+            if max_tools is not None:
+                successful_tool_calls = [
+                    h for h in history 
+                    if h.get("action") == "tool" 
+                    and "result" in h 
+                    and h["result"] not in [None, ""]
+                    and not str(h["result"]).startswith("Skipped:")
+                    and "error" not in h
+                ]
+                
+                if len(successful_tool_calls) >= max_tools:
+                    logger.warning(
+                        "[%s] Maximum tool calls limit reached (%d/%d). Stopping execution.",
+                        self.config.name,
+                        len(successful_tool_calls),
+                        max_tools
                     )
-                })
+                    return {
+                        "status": "success",
+                        "result": f"Task completed with {len(successful_tool_calls)} tool calls. Maximum tool call limit ({max_tools}) reached.",
+                        "iterations": iteration + 1,
+                        "history": history,
+                        "max_tool_calls_reached": True
+                    }
         
         # Max iterations reached
         return {
@@ -1141,6 +1273,192 @@ class Agent:
             # Tracing must never break runtime
             pass
     
+    async def _save_paused_context(self, context: PausedExecutionContext) -> None:
+        """
+        Save paused execution context to storage for later resumption.
+        
+        Uses cache backend for fast access with TTL-based cleanup.
+        Stores in memory table as backup for durability.
+        """
+        cache_key = f"paused_context:{context.delegation_task_id}"
+        context_data = context.to_dict()
+        
+        # Store in cache with TTL (2x timeout for safety margin)
+        ttl = context.timeout_seconds * 2
+        try:
+            await self._put_in_cache(cache_key, context_data, ttl=ttl)
+        except Exception as e:
+            logger.warning("Failed to save paused context to cache: %s", e)
+        
+        # Store in database memory table as durable backup
+        try:
+            self.memory.put(
+                f"paused_context:{context.delegation_task_id}",
+                context_data
+            )
+        except Exception as e:
+            logger.warning("Failed to save paused context to database: %s", e)
+        
+        # Trace the pause event
+        self._trace(
+            context.parent_task_id,
+            context.parent_agent_name,
+            "execution_paused",
+            {
+                "delegation_task_id": context.delegation_task_id,
+                "delegated_agent": context.delegated_agent_name,
+                "iteration": context.iteration,
+                "paused_at": context.paused_at,
+            }
+        )
+    
+    async def _load_paused_context(self, delegation_task_id: str) -> PausedExecutionContext | None:
+        """
+        Load paused execution context from storage.
+        
+        Tries cache first (fast), falls back to database if needed.
+        """
+        cache_key = f"paused_context:{delegation_task_id}"
+        
+        # Try cache first
+        try:
+            context_data = await self._get_from_cache(cache_key)
+            if context_data:
+                return PausedExecutionContext.from_dict(context_data)
+        except Exception as e:
+            logger.warning("Failed to load paused context from cache: %s", e)
+        
+        # Fall back to database
+        try:
+            context_data = self.memory.get(f"paused_context:{delegation_task_id}")
+            if context_data:
+                return PausedExecutionContext.from_dict(context_data)
+        except Exception as e:
+            logger.warning("Failed to load paused context from database: %s", e)
+        
+        return None
+    
+    async def _cleanup_paused_context(self, delegation_task_id: str) -> None:
+        """Clean up paused context after successful resumption."""
+        cache_key = f"paused_context:{delegation_task_id}"
+        
+        # Remove from cache
+        try:
+            if hasattr(self.cache_backend, 'delete'):
+                if asyncio.iscoroutinefunction(self.cache_backend.delete):
+                    await self.cache_backend.delete(cache_key)
+                else:
+                    self.cache_backend.delete(cache_key)
+        except Exception:
+            pass
+        
+        # Remove from database
+        try:
+            # Memory table doesn't have explicit delete, but we can set to None
+            self.memory.put(f"paused_context:{delegation_task_id}", None)
+        except Exception:
+            pass
+    
+    async def resume_paused_execution(
+        self,
+        delegation_task_id: str,
+        delegation_result: dict
+    ) -> dict:
+        """
+        Resume a paused agent execution after delegation completes.
+        
+        This is called by the orchestrator when a delegated task finishes.
+        It reloads the parent agent's saved context and continues execution
+        from where it left off, injecting the delegation result into history.
+        
+        Args:
+            delegation_task_id: The task_id of the completed delegation
+            delegation_result: The result returned by the delegated agent
+        
+        Returns:
+            Final execution result (may be paused again if another delegation occurs)
+        """
+        # Load paused context
+        context = await self._load_paused_context(delegation_task_id)
+        if not context:
+            logger.error(
+                "Cannot resume execution for delegation_task_id=%s: context not found",
+                delegation_task_id
+            )
+            return {
+                "status": "error",
+                "error": f"Paused context not found for delegation {delegation_task_id}",
+                "prompt_id": self.current_job_id
+            }
+        
+        # Trace resumption
+        from datetime import datetime as _dt
+        resumed_at = _dt.utcnow().isoformat() + "Z"
+        self._trace(
+            context.parent_task_id,
+            context.parent_agent_name,
+            "execution_resumed",
+            {
+                "delegation_task_id": delegation_task_id,
+                "delegated_agent": context.delegated_agent_name,
+                "resumed_at": resumed_at,
+                "delegation_result_status": delegation_result.get("status"),
+            }
+        )
+        
+        # Restore execution state
+        task = context.task
+        history = context.history
+        iteration = context.iteration
+        max_iterations = context.max_iterations
+        max_tool_calls = context.max_tool_calls
+        seen_queries = context.seen_queries
+        seen_delegations = context.seen_delegations
+        
+        # Inject delegation result into history
+        result_status = delegation_result.get("status")
+        result_data = delegation_result.get("result") or delegation_result.get("error")
+        
+        history.append({
+            "action": "delegate",
+            "agent": context.delegated_agent_name,
+            "task": context.delegated_task.get("query", ""),
+            "result": delegation_result,
+            "status": result_status
+        })
+        
+        # Check if delegation failed - stop to avoid retry loops
+        if result_status in ["error", "incomplete"]:
+            logger.warning(
+                "[%s] Delegated task to %s returned status=%s, stopping to avoid retry loop",
+                self.config.name,
+                context.delegated_agent_name,
+                result_status
+            )
+            await self._cleanup_paused_context(delegation_task_id)
+            return {
+                "status": "error",
+                "error": f"Delegated task to {context.delegated_agent_name} failed: {result_data}",
+                "iterations": iteration + 1,
+                "history": history,
+                "prompt_id": context.parent_task_id
+            }
+        
+        # Clean up paused context (we're resuming now)
+        await self._cleanup_paused_context(delegation_task_id)
+        
+        # Continue autonomous execution from next iteration
+        # We call the internal loop helper to avoid re-setup
+        return await self._autonomous_run_loop(
+            task=task,
+            history=history,
+            start_iteration=iteration + 1,  # Continue from next iteration
+            max_iterations=max_iterations,
+            max_tool_calls=max_tool_calls,
+            seen_queries=seen_queries,
+            seen_delegations=seen_delegations
+        )
+    
     def _build_autonomous_system_prompt(
         self,
         available_tools: list[str],
@@ -1180,7 +1498,7 @@ class Agent:
 You act autonomously to complete the task. Always respond with a SINGLE JSON object only (no prose, no code fences).
 Schema:
 {{
-  "type": "tool|delegate|finish|think",
+  "type": "tool|delegate|finish",
   "tool": "<tool_name_if_type_tool>",
   "params": {{"param1": "value1"}},
   "agent": "<agent_name_if_type_delegate>",
@@ -1192,7 +1510,7 @@ Schema:
 ## Available Tools:
 {tool_list}
 
-If you need to use a tool:
+To use a tool:
 {{"type":"tool","tool":"<tool_name>","params":{{"param":"value"}}}}
 """
 
@@ -1201,41 +1519,54 @@ If you need to use a tool:
 ## Available Agents (for delegation):
 {agent_list}
 
-If you need to delegate:
-{{"type":"delegate","agent":"<agent_name>","task":"<what should they do>","timeout":90}}
+To delegate a task:
+{{"type":"delegate","agent":"<agent_name>","task":"<clear task description>","timeout":90}}
 
-Rules for delegation:
-- Only use agent names from the list above; do NOT invent new agent names.
-- If none of the listed agents is appropriate, do not delegate; use tools or finish instead.
+⚠️ CRITICAL DELEGATION RULES:
+- ONLY use agent names from the list above - do NOT invent names
+- If no listed agent matches your needs, use tools or finish instead
+- Each agent has specific expertise (role/goal) - choose the most appropriate one
+- Delegation is for complex subtasks that require specialized processing
 
-## Parallel Processing (Sharding/Fan-out):
-When you have a large task that can be split into independent chunks:
+## Key System Tools:
 
-1. Use 'system_split_document' to divide content into sections:
-   {{"type":"tool","tool":"system_split_document","params":{{"document":"<text>","num_chunks":3}}}}
+### Parallel Processing (for independent subtasks):
+{{"type":"tool","tool":"system_delegate_parallel","params":{{"agent":"<agent_name>","tasks":["task1","task2","task3"]}}}}
+- Use when you have multiple independent subtasks that can run simultaneously
+- All tasks execute in parallel and results are aggregated
+- Ideal for: processing multiple documents, analyzing multiple data points, gathering info from multiple sources
 
-2. Then use 'system_delegate_parallel' to process all chunks simultaneously:
-   {{"type":"tool","tool":"system_delegate_parallel","params":{{"agent":"researcher","tasks":["analyze chunk 1","analyze chunk 2","analyze chunk 3"]}}}}
+### Document Splitting (for large content):
+{{"type":"tool","tool":"system_split_document","params":{{"document":"<text>","num_chunks":3}}}}
+- Use before parallel processing to divide large documents into manageable chunks
+- Returns list of chunks that can be passed to system_delegate_parallel
 
-This pattern is ideal for:
-- Processing large documents in sections
-- Analyzing multiple sources concurrently
-- Any task that can be divided into independent subtasks
+### Artifact Storage (for sharing data):
+{{"type":"tool","tool":"system_store_artifact","params":{{"key":"<unique_id>","value":"<data>","ttl":3600}}}}
+- Store results, intermediate data, or large outputs for other agents to retrieve
+- Use when delegating and the subtask needs access to data you've gathered
+- TTL is in seconds (default: 1 hour)
 
-The system will automatically wait for all parallel tasks to complete and return aggregated results.
+### Artifact Retrieval:
+{{"type":"tool","tool":"system_get_artifact","params":{{"key":"<unique_id>"}}}}
+- Retrieve previously stored artifacts
+- Useful when building on work from previous steps or other agents
 """
 
         system += """
-When you are done:
-{"type":"finish","answer":"<final comprehensive answer>"}
+When you complete the task:
+{{"type":"finish","answer":"<comprehensive final answer>"}}
 
-## Important Guidelines:
-- Avoid redundant tool calls - don't search for the same information multiple times
-- After gathering 2-3 reliable sources, you likely have enough information to finish
-- Quality over quantity - focus on synthesizing what you've found rather than endlessly searching
-- If you see a system hint about having sufficient information, strongly consider finishing
+## Execution Guidelines:
+- Use tools strategically - avoid redundant calls for the same information
+- Delegate complex subtasks to specialized agents when available
+- Use parallel processing for independent subtasks to improve efficiency
+- Store intermediate results as artifacts when they'll be reused
+- Focus on quality synthesis over quantity of actions
+- Finish when you have sufficient information to provide a comprehensive answer
+
+Think step-by-step internally, but output ONLY the JSON action object. No explanatory text."""
 
-Think step by step, but output JSON only. Do not include THOUGHT or any extra text; just the JSON object."""
 
         return system
     
@@ -1279,7 +1610,7 @@ Think step by step, but output JSON only. Do not include THOUGHT or any extra te
         prompt += f"What should you do next? (Iteration {iteration + 1})\n"
         
         return prompt
-    
+
     def _parse_autonomous_action(self, response: str) -> dict:
         """Parse LLM response to extract action, preferring JSON format with text fallback."""
         # 1) Try JSON extraction anywhere in the response
