@@ -51,7 +51,7 @@ def load_agents() -> Dict[str, Any]:
 Configuration management for EvenAge.
 
 Provides pluggable backend configuration via environment and YAML.
-All integrations (queue, database, vector, LLM, cache) are configurable.
+All integrations (queue, database, LLM, cache) are configurable.
 """
 
 from pathlib import Path
@@ -69,7 +69,6 @@ class EvenAgeConfig(BaseSettings):
     Supports pluggable backends for all infrastructure:
     - queue_backend: "redis" (default) | "memory"
     - db_backend: "postgres" (default) | "sqlite"
-    - vector_backend: "local" (default) | "pinecone" | "milvus"
     - llm_backend: "noop" (default) | "openai" | "anthropic" | "gemini"
     - cache_backend: "inmemory" (default) | "redis"
     """
@@ -77,7 +76,6 @@ class EvenAgeConfig(BaseSettings):
     # Pluggable backend selection
     queue_backend: str = Field(default="redis", description="Message queue backend")
     db_backend: str = Field(default="postgres", description="Database backend")
-    vector_backend: str = Field(default="local", description="Vector store backend")
     llm_backend: str = Field(default="noop", description="LLM backend (noop=echo)")
     llm_model: str | None = Field(default=None, description="LLM model name (optional, backend-specific)")
     openai_base_url: str | None = Field(default=None, description="OpenAI-compatible base URL (e.g., vLLM)")
@@ -124,28 +122,6 @@ class EvenAgeConfig(BaseSettings):
         default="evenage",
         description="Default storage bucket name"
     )
-    
-    # Backward compatibility aliases (deprecated - use storage_* instead)
-    minio_endpoint: str | None = Field(
-        default=None,
-        description="[DEPRECATED] Use storage_endpoint instead"
-    )
-    minio_access_key: str | None = Field(
-        default=None,
-        description="[DEPRECATED] Use storage_access_key instead"
-    )
-    minio_secret_key: str | None = Field(
-        default=None,
-        description="[DEPRECATED] Use storage_secret_key instead"
-    )
-    minio_secure: bool | None = Field(
-        default=None,
-        description="[DEPRECATED] Use storage_secure instead"
-    )
-    minio_bucket: str | None = Field(
-        default=None,
-        description="[DEPRECATED] Use storage_bucket instead"
-    )
 
     # Tracing and metrics (stored in DB, not external services)
     enable_tracing: bool = Field(default=True, description="Enable internal tracing to DB")
@@ -198,11 +174,11 @@ class EvenAgeConfig(BaseSettings):
         description="Name of the coordinator agent (used when has_coordinator=True)"
     )
     
-    # MCP (Model Context Protocol) Integration
-    mcp_servers: list[dict[str, str]] = Field(
-        default_factory=list,
-        description="List of MCP server configs with 'url' and optional 'api_key'"
-    )
+    # MCP (Model Context Protocol) Integration - DISABLED FOR THIS RELEASE
+    # mcp_servers: list[dict[str, str]] = Field(
+    #     default_factory=list,
+    #     description="List of MCP server configs with 'url' and optional 'api_key'"
+    # )
 
     # LLM API keys (optional, used when respective llm_backend is selected)
     gemini_api_key: str | None = Field(
@@ -223,12 +199,6 @@ class EvenAgeConfig(BaseSettings):
     grok_api_key: str | None = Field(
         default=None, description="xAI Grok API key (alias)"
     )
-    together_api_key: str | None = Field(
-        default=None, description="Together AI API key (for Meta Llama)"
-    )
-    replicate_api_token: str | None = Field(
-        default=None, description="Replicate API token (for Meta Llama)"
-    )
     http_llm_endpoint: str | None = Field(default=None, description="Generic HTTP LLM endpoint URL")
 
     # Tool API keys (optional)
@@ -249,7 +219,7 @@ class EvenAgeConfig(BaseSettings):
 
 # Backend protocols for pluggable implementations
 
-
+# keep these:- interface defs
 class QueueBackend(Protocol):
     """Protocol for message queue backends."""
 
@@ -277,13 +247,6 @@ class DatabaseBackend(Protocol):
     def list_agents(self) -> list[dict]: ...
 
 
-class VectorBackend(Protocol):
-    """Protocol for vector store backends."""
-
-    def store(self, key: str, vector: list[float], metadata: dict) -> None: ...
-    def search(self, vector: list[float], k: int) -> list[dict]: ...
-    def delete(self, key: str) -> None: ...
-
 
 class LLMBackend(Protocol):
     """Protocol for LLM backends."""
@@ -304,7 +267,7 @@ class BackendFactory:
     """
     Factory for creating backend instances based on configuration.
     
-    Provides pluggable implementations for queue, database, vector, LLM, and cache.
+    Provides pluggable implementations for queue, database, LLM, and cache.
     """
 
     def __init__(self, config: EvenAgeConfig):
@@ -388,15 +351,6 @@ class BackendFactory:
         if self.config.llm_backend == "grok":
             from .llm import GrokLLM
             return GrokLLM(self.config.xai_api_key or self.config.grok_api_key, self.config.llm_model)
-        if self.config.llm_backend == "meta":
-            from .llm import MetaLLM
-            # Default to Together AI, fallback to Replicate if Together key not set
-            provider = "together" if self.config.together_api_key else "replicate"
-            return MetaLLM(
-                self.config.together_api_key or self.config.replicate_api_token,
-                self.config.llm_model,
-                provider=provider
-            )
         if self.config.llm_backend == "http":
             from .llm import HTTPLLM
             if not self.config.http_llm_endpoint:
@@ -431,13 +385,9 @@ class BackendFactory:
             if self.config.storage_endpoint != "localhost:9000" or self.config.minio_endpoint is None
             else self.config.minio_endpoint or self.config.storage_endpoint
         )
-        access_key = self.config.minio_access_key or self.config.storage_access_key
-        secret_key = self.config.minio_secret_key or self.config.storage_secret_key
-        secure = (
-            self.config.minio_secure
-            if self.config.minio_secure is not None
-            else self.config.storage_secure
-        )
+        access_key = self.config.storage_access_key
+        secret_key = self.config.storage_secret_key
+        secure = self.config.storage_secure
         region = self.config.storage_region
         
         if endpoint:
@@ -467,7 +417,6 @@ class ProjectConfig(BaseModel):
     # Backend selections (can override env defaults)
     queue_backend: str = Field(default="redis", description="Message queue backend")
     db_backend: str = Field(default="postgres", description="Database backend")
-    vector_backend: str = Field(default="local", description="Vector store backend")
     llm_backend: str = Field(default="noop", description="LLM backend")
     cache_backend: str = Field(default="inmemory", description="Cache backend")
 
