@@ -163,14 +163,25 @@ class RedisBus:
             threshold_kb = getattr(self, "_storage_threshold_kb", None)
 
             payload_bytes = json.dumps(response).encode("utf-8")
-            if storage and threshold_kb is not None and len(payload_bytes) > threshold_kb * 1024:
+            # If threshold is 0, store all responses; otherwise check size
+            should_offload = (
+                storage and 
+                threshold_kb is not None and 
+                (threshold_kb == 0 or len(payload_bytes) > threshold_kb * 1024)
+            )
+            
+            print(f"[STORAGE] Checking offload: storage={storage is not None}, threshold={threshold_kb}, size={len(payload_bytes)}, should_offload={should_offload}")
+            
+            if should_offload:
                 # Build a key with date prefix for grouping
                 date_prefix = datetime.utcnow().strftime("%Y/%m/%d")
                 key = f"responses/{date_prefix}/{task_id}.json"
                 metadata = {"content-type": "application/json"}
                 try:
+                    print(f"[STORAGE] Offloading to {bucket}/{key} (size={len(payload_bytes)} bytes)")
                     await storage.ensure_bucket(bucket)
                     await storage.put_object(bucket, key, payload_bytes, metadata=metadata)
+                    print(f"[STORAGE] ✓ Successfully offloaded to {bucket}/{key}")
                     response = {
                         "task_id": task_id,
                         "offloaded": True,
@@ -178,10 +189,14 @@ class RedisBus:
                         "key": key,
                         "size_bytes": len(payload_bytes),
                     }
-                except Exception:
+                except Exception as e:
                     # Fallback: keep inline if storage fails
+                    print(f"[STORAGE] ✗ Failed to offload: {e}")
+                    import traceback
+                    traceback.print_exc()
                     pass
-        except Exception:
+        except Exception as e:
+            print(f"[STORAGE] Error in offload logic: {e}")
             pass
 
         # Store response
@@ -389,7 +404,14 @@ class MemoryBus:
             bucket = getattr(self, "_storage_bucket", None) or "evenage"
             threshold_kb = getattr(self, "_storage_threshold_kb", None)
             payload_bytes = json.dumps(response).encode("utf-8")
-            if storage and threshold_kb is not None and len(payload_bytes) > threshold_kb * 1024:
+            # If threshold is 0, store all responses; otherwise check size
+            should_offload = (
+                storage and 
+                threshold_kb is not None and 
+                (threshold_kb == 0 or len(payload_bytes) > threshold_kb * 1024)
+            )
+            
+            if should_offload:
                 date_prefix = datetime.utcnow().strftime("%Y/%m/%d")
                 key = f"responses/{date_prefix}/{task_id}.json"
                 metadata = {"content-type": "application/json"}
@@ -483,16 +505,40 @@ class KafkaBus:
             raise RuntimeError("aiokafka is not installed. Install with: pip install aiokafka")
         self.bootstrap_servers = bootstrap_servers
         self._producer = None
+        self._consumers: dict[str, Any] = {}  # Cache consumers per agent
         self._agents: dict[str, dict] = {}
         self._waiters: dict[str, list[asyncio.Future]] = {}
         self._canceled: set[str] = set()
 
     async def _get_producer(self) -> Any:
         if self._producer is None:
-            self._producer = AIOKafkaProducer(bootstrap_servers=self.bootstrap_servers)
+            print(f"[KafkaBus] Creating Kafka producer for {self.bootstrap_servers}")
+            self._producer = AIOKafkaProducer(
+                bootstrap_servers=self.bootstrap_servers,
+                acks='all',  # Wait for all replicas to acknowledge
+                max_request_size=10485760,  # 10MB max message size
+            )
             await self._producer.start()
+            print(f"[KafkaBus] Producer started successfully")
         return self._producer
 
+    async def _get_consumer(self, agent_name: str) -> Any:
+        """Get or create a persistent consumer for the agent."""
+        if agent_name not in self._consumers:
+            print(f"[KafkaBus] Creating consumer for agent: {agent_name}")
+            consumer = AIOKafkaConsumer(
+                f"evenage.tasks.{agent_name}",
+                bootstrap_servers=self.bootstrap_servers,
+                group_id=f"evenage-{agent_name}-workers",
+                enable_auto_commit=True,  # Auto-commit offsets
+                auto_offset_reset="earliest",  # Start from beginning if no committed offset
+                max_poll_records=10,
+            )
+            await consumer.start()
+            print(f"[KafkaBus] Consumer started for agent: {agent_name}")
+            self._consumers[agent_name] = consumer
+        return self._consumers[agent_name]
+
     async def register_agent(self, name: str, metadata: dict) -> bool:
         metadata["last_heartbeat"] = time.time()
         metadata["status"] = metadata.get("status", "active")
@@ -500,27 +546,49 @@ class KafkaBus:
         return True
 
     async def publish_task(self, agent_name: str, task: dict) -> str:
-        producer = await self._get_producer()
-        task_id = str(uuid.uuid4())
-        message = TaskMessage(task_id=task_id, agent_name=agent_name, payload=task).to_dict()
-        data = json.dumps(message).encode("utf-8")
-        await producer.send_and_wait(f"evenage.tasks.{agent_name}", data)
-        return task_id
+        try:
+            print(f"[KafkaBus] Publishing task to agent: {agent_name}")
+            producer = await self._get_producer()
+            print(f"[KafkaBus] Producer obtained")
+            task_id = str(uuid.uuid4())
+            message = TaskMessage(task_id=task_id, agent_name=agent_name, payload=task).to_dict()
+            data = json.dumps(message).encode("utf-8")
+            topic = f"evenage.tasks.{agent_name}"
+            print(f"[KafkaBus] Sending to topic: {topic}, task_id: {task_id}")
+            await producer.send_and_wait(topic, data)
+            print(f"[KafkaBus] Task published successfully: {task_id}")
+            return task_id
+        except Exception as e:
+            print(f"[KafkaBus] ERROR publishing task: {e}")
+            import traceback
+            traceback.print_exc()
+            raise
 
     async def publish_response(self, task_id: str, response: dict) -> bool:
+        print(f"[KafkaBus] Publishing response for task: {task_id}")
+        
         # Optional offload similar to other buses
         try:
             storage = getattr(self, "_storage", None)
             bucket = getattr(self, "_storage_bucket", None) or "evenage"
             threshold_kb = getattr(self, "_storage_threshold_kb", None)
             payload_bytes = json.dumps(response).encode("utf-8")
-            if storage and threshold_kb is not None and len(payload_bytes) > threshold_kb * 1024:
+            # If threshold is 0, store all responses; otherwise check size
+            should_offload = (
+                storage and 
+                threshold_kb is not None and 
+                (threshold_kb == 0 or len(payload_bytes) > threshold_kb * 1024)
+            )
+            
+            if should_offload:
+                print(f"[KafkaBus] Offloading response to storage (size: {len(payload_bytes)} bytes)")
                 date_prefix = datetime.utcnow().strftime("%Y/%m/%d")
                 key = f"responses/{date_prefix}/{task_id}.json"
                 metadata = {"content-type": "application/json"}
                 try:
                     await storage.ensure_bucket(bucket)
                     await storage.put_object(bucket, key, payload_bytes, metadata=metadata)
+                    print(f"[KafkaBus] Response offloaded to {bucket}/{key}")
                     response = {
                         "task_id": task_id,
                         "offloaded": True,
@@ -528,9 +596,11 @@ class KafkaBus:
                         "key": key,
                         "size_bytes": len(payload_bytes),
                     }
-                except Exception:
+                except Exception as e:
+                    print(f"[KafkaBus] ERROR offloading response: {e}")
                     pass
-        except Exception:
+        except Exception as e:
+            print(f"[KafkaBus] ERROR in publish_response storage check: {e}")
             pass
 
         # Notify in-memory waiters
@@ -542,22 +612,17 @@ class KafkaBus:
 
         # Fire-and-forget to responses topic for external consumers
         try:
+            print(f"[KafkaBus] Publishing to evenage.responses topic...")
             producer = await self._get_producer()
             await producer.send_and_wait("evenage.responses", json.dumps(response).encode("utf-8"))
-        except Exception:
+            print(f"[KafkaBus] Response published to Kafka successfully")
+        except Exception as e:
+            print(f"[KafkaBus] ERROR publishing response to Kafka: {e}")
             pass
         return True
 
     async def consume_tasks(self, agent_name: str, block_ms: int, count: int) -> list[dict]:
-        consumer = AIOKafkaConsumer(  # type: ignore[name-defined]
-            f"evenage.tasks.{agent_name}",
-            bootstrap_servers=self.bootstrap_servers,
-            enable_auto_commit=False,
-            auto_offset_reset="latest",
-            max_poll_records=count,
-            request_timeout_ms=max(block_ms, 1000),
-        )
-        await consumer.start()
+        consumer = await self._get_consumer(agent_name)
         tasks: list[dict] = []
         try:
             result = await consumer.getmany(timeout_ms=block_ms, max_records=count)
@@ -571,22 +636,119 @@ class KafkaBus:
             # commit offsets for consumed messages
             if tasks:
                 await consumer.commit()
-        finally:
-            await consumer.stop()
+        except Exception as e:
+            # Log error but don't crash the worker
+            print(f"Error consuming tasks: {e}")
         return tasks
 
     async def wait_for_response(self, task_id: str, timeout_sec: int) -> dict | None:
-        # Minimal in-memory waiter; external consumers can listen on topic
-        if task_id in getattr(self, "_responses", {}):
-            return self._responses[task_id]
-        fut = asyncio.Future()
-        self._waiters.setdefault(task_id, []).append(fut)
+        """
+        Wait for a response from Kafka responses topic.
+        For Kafka, we need to actually poll the topic since in-memory waiters
+        don't work across processes.
+        """
+        print(f"[KafkaBus] Waiting for response to task: {task_id}, timeout: {timeout_sec}s")
+        
+        # Create a dedicated consumer for this response
+        # Use a unique group ID so we read all messages
+        consumer = AIOKafkaConsumer(
+            "evenage.responses",
+            bootstrap_servers=self.bootstrap_servers,
+            group_id=f"wait-{task_id}",  # Unique group per wait
+            auto_offset_reset="earliest",  # Read all messages
+            enable_auto_commit=False,
+            consumer_timeout_ms=1000,  # Poll every second
+        )
+        
         try:
-            return await asyncio.wait_for(fut, timeout=timeout_sec)
-        except asyncio.TimeoutError:
-            return None
+            await consumer.start()
+            print(f"[KafkaBus] Consumer started, polling for response...")
+            
+            start_time = asyncio.get_event_loop().time()
+            while True:
+                # Check timeout
+                elapsed = asyncio.get_event_loop().time() - start_time
+                if elapsed >= timeout_sec:
+                    print(f"[KafkaBus] Timeout waiting for response to {task_id}")
+                    return None
+                
+                # Poll for messages
+                try:
+                    result = await consumer.getmany(timeout_ms=2000, max_records=100)
+                    for tp, messages in result.items():
+                        for msg in messages:
+                            try:
+                                data = json.loads(msg.value.decode("utf-8"))
+                                response_task_id = data.get("task_id")
+                                
+                                if response_task_id == task_id:
+                                    print(f"[KafkaBus] Found response for task {task_id}")
+                                    
+                                    # Check if response was offloaded to storage
+                                    if data.get("offloaded"):
+                                        print(f"[KafkaBus] Response offloaded, retrieving from storage...")
+                                        # Retrieve from storage
+                                        storage = getattr(self, "_storage", None)
+                                        if storage:
+                                            bucket = data.get("bucket", "evenage")
+                                            key = data.get("key")
+                                            if key:
+                                                try:
+                                                    stored_data = await storage.get_object(bucket, key)
+                                                    actual_response = json.loads(stored_data.decode("utf-8"))
+                                                    print(f"[KafkaBus] Retrieved response from storage")
+                                                    return actual_response
+                                                except Exception as e:
+                                                    print(f"[KafkaBus] Error retrieving from storage: {e}")
+                                                    return data  # Return offload metadata as fallback
+                                        return data
+                                    else:
+                                        # Response is inline
+                                        return data
+                            except Exception as e:
+                                print(f"[KafkaBus] Error parsing message: {e}")
+                                continue
+                except asyncio.TimeoutError:
+                    # No messages in this poll, continue waiting
+                    pass
+                
+                # Small sleep to avoid tight loop
+                await asyncio.sleep(0.1)
+                
+        finally:
+            await consumer.stop()
+            print(f"[KafkaBus] Consumer stopped for task {task_id}")
 
     async def get_registered_agents(self) -> dict[str, dict]:
+        """
+        Get registered agents by discovering Kafka topics.
+        Topics follow the pattern: evenage.tasks.<agent_name>
+        """
+        try:
+            from aiokafka.admin import AIOKafkaAdminClient
+            admin = AIOKafkaAdminClient(bootstrap_servers=self.bootstrap_servers)
+            await admin.start()
+            try:
+                topics = await admin.list_topics()
+                # Extract agent names from topic names
+                prefix = "evenage.tasks."
+                for topic in topics:
+                    if topic.startswith(prefix):
+                        agent_name = topic[len(prefix):]
+                        if agent_name not in self._agents:
+                            # Register discovered agent with default metadata
+                            self._agents[agent_name] = {
+                                "role": "agent",
+                                "goal": f"Process tasks for {agent_name}",
+                                "status": "active",
+                                "last_heartbeat": time.time(),
+                                "tools": []
+                            }
+            finally:
+                await admin.close()
+        except Exception:
+            # If discovery fails, return what we have in memory
+            pass
         return dict(self._agents)
 
     async def list_agents(self) -> list[dict]:
