@@ -135,6 +135,36 @@ class AgentRunner:
 
             # Handle task
             result = await agent.handle(task)
+            
+            # Check if execution was PAUSED (waiting for delegation)
+            if result.get("status") == "paused":
+                delegation_task_id = result.get("delegation_task_id")
+                delegated_agent = result.get("delegated_agent")
+                
+                # Store paused state in job metadata
+                self.database.save_result(
+                    job_id=job_id,
+                    outputs={
+                        "status": "paused",
+                        "delegation_task_id": delegation_task_id,
+                        "delegated_agent": delegated_agent,
+                        "message": result.get("message", "Execution paused")
+                    },
+                    status="paused"
+                )
+                
+                # Set up delegation response listener
+                # When delegation completes, orchestrator should call handle_delegation_response()
+                duration_ms = int((time.time() - start) * 1000)
+                return {
+                    "job_id": job_id,
+                    "status": "paused",
+                    "delegation_task_id": delegation_task_id,
+                    "delegated_agent": delegated_agent,
+                    "message": f"Execution paused, waiting for {delegated_agent}",
+                    "duration_ms": duration_ms,
+                    "agent": pipeline_name
+                }
 
             # Save result
             outputs = result.get("result", {})
@@ -183,6 +213,159 @@ class AgentRunner:
             if self._inline_worker_task is not None:
                 with suppress(Exception):
                     self._stop_inline_workers()
+    
+    async def handle_delegation_response(
+        self,
+        delegation_task_id: str,
+        delegation_result: dict
+    ) -> dict:
+        """
+        Handle a completed delegation and resume parent agent execution.
+        
+        This is called by the orchestrator when a delegated task completes.
+        It loads the parent agent, resumes from saved context, and continues execution.
+        
+        Args:
+            delegation_task_id: The task_id of the completed delegation
+            delegation_result: The result returned by the delegated agent
+        
+        Returns:
+            Result from resumed execution (may be paused again if chained delegations)
+        
+        Example orchestrator pattern:
+            ```python
+            # 1. Run parent agent
+            result = await runner.run(inputs, agent_name="coordinator")
+            
+            # 2. If paused, wait for delegation to complete
+            if result["status"] == "paused":
+                delegation_task_id = result["delegation_task_id"]
+                
+                # Listen for delegation completion (via message bus or polling)
+                delegation_result = await wait_for_delegation(delegation_task_id)
+                
+                # 3. Resume parent execution
+                final_result = await runner.handle_delegation_response(
+                    delegation_task_id,
+                    delegation_result
+                )
+            ```
+        """
+        import logging
+        logger = logging.getLogger(__name__)
+        
+        # Load the parent agent from paused context
+        # We need to find which agent was paused - check cache/memory
+        bus = self.factory.create_queue_backend()
+        cache = self.factory.create_cache_backend()
+        
+        # Try to load context from cache to identify parent agent
+        cache_key = f"paused_context:{delegation_task_id}"
+        context_data = None
+        
+        try:
+            if hasattr(cache, 'get') and asyncio.iscoroutinefunction(cache.get):
+                context_data = await cache.get(cache_key)
+            else:
+                context_data = cache.get(cache_key)
+        except Exception as e:
+            logger.warning(f"Failed to load context from cache: {e}")
+        
+        if not context_data:
+            return {
+                "status": "error",
+                "error": f"Cannot resume: paused context not found for delegation {delegation_task_id}"
+            }
+        
+        parent_agent_name = context_data.get("parent_agent_name")
+        parent_task_id = context_data.get("parent_task_id")
+        
+        logger.info(
+            f"Resuming agent '{parent_agent_name}' from delegation_task_id={delegation_task_id}"
+        )
+        
+        # Load the parent agent
+        try:
+            mod = importlib.import_module(f"agents.{parent_agent_name}")
+            agent = getattr(mod, parent_agent_name, None)
+            if not agent or not hasattr(agent, "resume_paused_execution"):
+                raise AttributeError(f"Agent {parent_agent_name} not found or doesn't support resumption")
+        except Exception as e:
+            logger.error(f"Failed to load parent agent '{parent_agent_name}': {e}")
+            return {
+                "status": "error",
+                "error": f"Failed to load parent agent: {e}"
+            }
+        
+        # Ensure bus connection
+        try:
+            await agent.connect_bus()
+        except Exception:
+            pass
+        
+        # Resume execution
+        start = time.time()
+        try:
+            result = await agent.resume_paused_execution(
+                delegation_task_id,
+                delegation_result
+            )
+            
+            # Check if paused again (chained delegations)
+            if result.get("status") == "paused":
+                logger.info(
+                    f"Agent '{parent_agent_name}' paused again at delegation_task_id={result.get('delegation_task_id')}"
+                )
+                # Update job with new paused state
+                self.database.save_result(
+                    job_id=parent_task_id,
+                    outputs={
+                        "status": "paused",
+                        "delegation_task_id": result.get("delegation_task_id"),
+                        "delegated_agent": result.get("delegated_agent"),
+                        "message": result.get("message", "Execution paused again")
+                    },
+                    status="paused"
+                )
+                return result
+            
+            # Execution completed
+            outputs = result.get("result", {})
+            status = result.get("status", "completed")
+            error = result.get("error")
+            
+            if error:
+                outputs = {"error": error, **outputs}
+            
+            self.database.save_result(
+                job_id=parent_task_id,
+                outputs=outputs,
+                status=status
+            )
+            
+            duration_ms = int((time.time() - start) * 1000)
+            return {
+                "job_id": parent_task_id,
+                "status": status,
+                "result": outputs,
+                "error": error,
+                "duration_ms": duration_ms,
+                "agent": parent_agent_name
+            }
+            
+        except Exception as e:
+            logger.exception(f"Error resuming agent '{parent_agent_name}': {e}")
+            self.database.save_result(
+                job_id=parent_task_id,
+                outputs={"error": str(e)},
+                status="failed"
+            )
+            return {
+                "job_id": parent_task_id,
+                "status": "failed",
+                "error": str(e),
+                "agent": parent_agent_name
+            }
 
     def _discover_local_agents(self, exclude: list[str] | None = None) -> list[str]:
         """Discover local agents under the 'agents' package for inline execution."""
@@ -521,11 +704,6 @@ class WorkerRunner:
         storage_secret_key: str | None = None,
         storage_secure: bool | None = None,
         storage_region: str | None = None,
-        # Backward compatibility
-        minio_endpoint: str | None = None,
-        minio_access_key: str | None = None,
-        minio_secret_key: str | None = None,
-        minio_secure: bool | None = None,
     ) -> None:
         # Build env config from provided values (fallback to defaults)
         # Support both new storage_* and old minio_* parameter names
@@ -533,11 +711,11 @@ class WorkerRunner:
         self.env_config = EvenAgeConfig(
             database_url=database_url or defaults.database_url,
             redis_url=redis_url or defaults.redis_url,
-            storage_endpoint=storage_endpoint or minio_endpoint or defaults.storage_endpoint,
-            storage_access_key=storage_access_key or minio_access_key or defaults.storage_access_key,
-            storage_secret_key=storage_secret_key or minio_secret_key or defaults.storage_secret_key,
+            storage_endpoint=storage_endpoint or defaults.storage_endpoint,
+            storage_access_key=storage_access_key or defaults.storage_access_key,
+            storage_secret_key=storage_secret_key or defaults.storage_secret_key,
             storage_secure=storage_secure if storage_secure is not None else (
-                minio_secure if minio_secure is not None else defaults.storage_secure
+                False
             ),
             storage_region=storage_region or defaults.storage_region,
             storage_bucket=defaults.storage_bucket,  # Read from env
