@@ -587,7 +587,7 @@ class ParallelDelegationTool:
         }
 
 
-def create_system_tools(message_bus, storage_backend=None, agent=None) -> dict[str, Any]:
+def create_system_tools(message_bus, storage_backend=None, agent=None) -> dict[str, tuple[Any, list[str]]]:
     """
     Create system tools for task delegation and artifact management.
     
@@ -597,28 +597,33 @@ def create_system_tools(message_bus, storage_backend=None, agent=None) -> dict[s
         agent: Optional agent instance for accessing current_job_id context
     
     Returns:
-        Dict mapping tool names to tool instances
+        Dict mapping tool names to (tool_function, aliases) tuples
     """
     tools = {}
 
     # Task delegation tool (with agent context for job_id propagation)
     delegation_tool = TaskDelegationTool(message_bus, storage_backend, agent=agent)
-    tools["system_delegate_task"] = delegation_tool.delegate_task
+    tools["system_delegate_task"] = (delegation_tool.delegate_task, [])
 
-    # Parallel delegation and document splitting tools
+    # Parallel delegation and document splitting tools WITH ALIASES
     parallel_tool = ParallelDelegationTool(message_bus, storage_backend, agent=agent)
-    # System-prefixed names
-    tools["system_delegate_parallel"] = parallel_tool.delegate_parallel
-    tools["system_split_document"] = parallel_tool.split_document
-    # Friendly aliases used frequently by LLMs/examples
-    tools["delegate_parallel"] = parallel_tool.delegate_parallel
-    tools["split_document"] = parallel_tool.split_document
+    
+    # Register with aliases instead of duplicate entries
+    tools["system_delegate_parallel"] = (
+        parallel_tool.delegate_parallel,
+        ["delegate_parallel"]  # Friendly alias
+    )
+    
+    tools["system_split_document"] = (
+        parallel_tool.split_document,
+        ["split_document"]  # Friendly alias
+    )
 
     # Artifact storage tools (if storage backend available)
     if storage_backend:
         artifact_tool = ArtifactStorageTool(storage_backend)
-        tools["system_store_artifact"] = artifact_tool.store_artifact
-        tools["system_retrieve_artifact"] = artifact_tool.retrieve_artifact
-        tools["system_list_artifacts"] = artifact_tool.list_artifacts
+        tools["system_store_artifact"] = (artifact_tool.store_artifact, [])
+        tools["system_retrieve_artifact"] = (artifact_tool.retrieve_artifact, [])
+        tools["system_list_artifacts"] = (artifact_tool.list_artifacts, [])
 
     return tools
