@@ -140,44 +140,78 @@ class ToolRegistry:
 
     def __init__(self):
         self._tools: dict[str, Tool] = {}
+        self._aliases: dict[str, str] = {}  # alias -> canonical name mapping
 
-    def register(self, tool_obj: Tool | Callable) -> None:
+    def register(self, tool_obj: Tool | Callable, name: str | None = None, aliases: list[str] | None = None) -> None:
         """
-        Register a tool.
+        Register a tool with optional aliases.
         
-        Accepts either a Tool object or a decorated callable.
+        Args:
+            tool_obj: Tool instance or callable to register
+            name: Optional explicit name (overrides tool_obj.name)
+            aliases: Optional list of alias names that resolve to this tool
         """
+        # Determine the canonical name and tool object
         if isinstance(tool_obj, Tool):
-            self._tools[tool_obj.name] = tool_obj
+            tool = tool_obj
+            canonical_name = name or tool.name
+            # Update the tool's name if explicit name provided
+            if name and name != tool.name:
+                tool = Tool(name=name, func=tool.func, description=tool.description)
         elif hasattr(tool_obj, "__evenage_tool__"):
             tool = tool_obj.__evenage_tool__
-            self._tools[tool.name] = tool
+            canonical_name = name or tool.name
+            # Update the tool's name if explicit name provided
+            if name and name != tool.name:
+                tool = Tool(name=name, func=tool.func, description=tool.description)
         elif callable(tool_obj):
             # Auto-wrap callable without decorator
+            canonical_name = name or tool_obj.__name__
             tool = Tool(
-                name=tool_obj.__name__,
+                name=canonical_name,
                 func=tool_obj,
                 description=(tool_obj.__doc__ or "").strip().split("\n")[0]
             )
-            self._tools[tool.name] = tool
         else:
             raise ValueError(f"Cannot register {tool_obj} as a tool")
 
+        # Register the tool under its canonical name
+        self._tools[canonical_name] = tool
+
+        # Register aliases if provided
+        if aliases:
+            for alias in aliases:
+                self._aliases[alias] = canonical_name
+
     def get(self, name: str) -> Tool | None:
-        """Get a tool by name."""
-        return self._tools.get(name)
+        """
+        Get a tool by name or alias.
+        
+        Args:
+            name: Tool name (canonical or alias)
+        
+        Returns:
+            Tool instance or None if not found
+        """
+        # Check if name is an alias first
+        canonical_name = self._aliases.get(name, name)
+        return self._tools.get(canonical_name)
 
     def list(self) -> list[Tool]:
-        """List all registered tools."""
+        """List all registered tools (excludes aliases)."""
         return list(self._tools.values())
 
     def list_names(self) -> list[str]:
-        """List all tool names."""
+        """List all canonical tool names (excludes aliases)."""
         return list(self._tools.keys())
 
+    def list_all_names(self) -> list[str]:
+        """List all names including aliases (for debugging)."""
+        return list(self._tools.keys()) + list(self._aliases.keys())
+
     def has(self, name: str) -> bool:
-        """Check if a tool exists."""
-        return name in self._tools
+        """Check if a tool exists (checks both canonical names and aliases)."""
+        return name in self._tools or name in self._aliases
 
 
 def discover_tools(agent_name: str) -> ToolRegistry:
@@ -275,55 +309,56 @@ def bind_tools(agent_instance: Any, tools: list[str | Callable]) -> None:
             raise ValueError(f"Tool must be string or callable, got {type(tool_ref)}")
 
 
-def register_mcp_tools(agent_instance: Any, mcp_source: Any) -> None:
-    """
-    Register MCP-discovered tools to an agent instance.
-    
-    Fetches tools from MCPToolSource and wraps them as callables.
-    
-    Usage:
-        from evenage.core.mcp_client import MCPToolSource
-        mcp = MCPToolSource("http://localhost:3000", api_key="...")
-        register_mcp_tools(self, mcp)
-    
-    Args:
-        agent_instance: Agent instance with .tools registry
-        mcp_source: MCPToolSource instance
-    """
-    import asyncio
-    
-    if not hasattr(agent_instance, "tools"):
-        raise AttributeError("Agent must have .tools ToolRegistry")
-    
-    # Discover tools from MCP server
-    try:
-        tools_metadata = asyncio.run(mcp_source.discover())
-    except Exception as e:
-        raise RuntimeError(f"Failed to discover MCP tools: {e}")
-    
-    # Wrap each MCP tool as a callable
-    for tool_meta in tools_metadata:
-        tool_name = tool_meta.get("name")
-        tool_desc = tool_meta.get("description", "")
-        
-        # Create async wrapper
-        async def mcp_tool_wrapper(**kwargs):
-            return await mcp_source.call_tool(tool_name, kwargs)
-        
-        # Create sync wrapper for compatibility
-        def sync_wrapper(**kwargs):
-            return asyncio.run(mcp_tool_wrapper(**kwargs))
-        
-        sync_wrapper.__name__ = tool_name
-        sync_wrapper.__doc__ = tool_desc
-        
-        # Register as tool
-        tool_obj = Tool(
-            name=tool_name,
-            func=sync_wrapper,
-            description=tool_desc
-        )
-        agent_instance.tools.register(tool_obj)
+# MCP feature disabled for this release
+# def register_mcp_tools(agent_instance: Any, mcp_source: Any) -> None:
+#     """
+#     Register MCP-discovered tools to an agent instance.
+#     
+#     Fetches tools from MCPToolSource and wraps them as callables.
+#     
+#     Usage:
+#         from evenage.core.mcp_client import MCPToolSource
+#         mcp = MCPToolSource("http://localhost:3000", api_key="...")
+#         register_mcp_tools(self, mcp)
+#     
+#     Args:
+#         agent_instance: Agent instance with .tools registry
+#         mcp_source: MCPToolSource instance
+#     """
+#     import asyncio
+#     
+#     if not hasattr(agent_instance, "tools"):
+#         raise AttributeError("Agent must have .tools ToolRegistry")
+#     
+#     # Discover tools from MCP server
+#     try:
+#         tools_metadata = asyncio.run(mcp_source.discover())
+#     except Exception as e:
+#         raise RuntimeError(f"Failed to discover MCP tools: {e}")
+#     
+#     # Wrap each MCP tool as a callable
+#     for tool_meta in tools_metadata:
+#         tool_name = tool_meta.get("name")
+#         tool_desc = tool_meta.get("description", "")
+#         
+#         # Create async wrapper
+#         async def mcp_tool_wrapper(**kwargs):
+#             return await mcp_source.call_tool(tool_name, kwargs)
+#         
+#         # Create sync wrapper for compatibility
+#         def sync_wrapper(**kwargs):
+#             return asyncio.run(mcp_tool_wrapper(**kwargs))
+#         
+#         sync_wrapper.__name__ = tool_name
+#         sync_wrapper.__doc__ = tool_desc
+#         
+#         # Register as tool
+#         tool_obj = Tool(
+#             name=tool_name,
+#             func=sync_wrapper,
+#             description=tool_desc
+#         )
+#         agent_instance.tools.register(tool_obj)
 
 
 def create_tool_schema(tool: Tool) -> dict[str, Any]:
